"""django_temp_permissions model validators."""

from datetime import datetime, timedelta

from django.conf import settings
from django.utils.translation import gettext as _
from django.core.exceptions import ValidationError
from django.utils import timezone


def validate_date_not_in_past(value: datetime) -> None:
    """Validate that a date is not in the past.

    If required the LENIENCE setting allows the extension
    of the time within the date will still be valid.
    By default, it's 5 minutes.
    """
    lenience = getattr(settings, "PAST_PERMISSIONS_LENIENCE", timedelta(minutes=5))

    if value <= timezone.now() - lenience:
        raise ValidationError(_("{value} date is in the past.").format(value=value))



# Generated by Django 5.2.6 on 2025-10-14 19:52

import django.db.models.deletion
import django_temp_permissions.validators
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ('auth', '0012_alter_user_first_name_max_length'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='TemporaryPermission',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                (
                    'start_datetime',
                    models.DateTimeField(
                        help_text='Date and time when the permission becomes active',
                        validators=[django_temp_permissions.validators.validate_date_not_in_past],
                    ),
                ),
                (
                    'end_datetime',
                    models.DateTimeField(
                        help_text='Date and time when the permission expires',
                        validators=[django_temp_permissions.validators.validate_date_not_in_past],
                    ),
                ),
                (
                    'notes',
                    models.TextField(
                        blank=True, default='', help_text='Optional notes about this temporary permission'
                    ),
                ),
                (
                    'permissions',
                    models.ManyToManyField(
                        help_text='The permission being granted temporarily',
                        related_name='temporary_permissions',
                        to='auth.permission',
                    ),
                ),
                (
                    'user',
                    models.ForeignKey(
                        help_text='User receiving the temporary permission',
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name='temporary_permissions',
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



"""Django temporary permission custom backend."""

from django.contrib.auth import get_user_model
from django.contrib.auth.backends import BaseBackend

from django_temp_permissions.models import TemporaryPermission

User = get_user_model()


class TemporaryPermissionBackend(BaseBackend):
    """Custom permission backend for temporary permissions support."""

    def get_user_permissions(self, user_obj: User, obj: str | None = None) -> set:
        """Append the temporary permissions to the list of permissions."""
        permissions = TemporaryPermission.objects.get_active_permissions(user_obj)
        perms = permissions.values_list("content_type__app_label", "codename").order_by()

        return {"%s.%s" % (ct, name) for ct, name in perms}



"""django-temp-permissions app config."""

from django.apps import AppConfig


class Config(AppConfig):  # noqa D101
    verbose_name = "Django Temporary Permissions"
    name = "django_temp_permissions"



"""Admin configuration for django-temp-permissions."""

from typing import Any

from admin_extra_buttons.decorators import button
from admin_extra_buttons.mixins import ExtraButtonsMixin
from adminfilters.dates import DateRangeFilter
from adminfilters.mixin import AdminFiltersMixin
from django.contrib import admin
from django.contrib.auth.base_user import AbstractBaseUser
from django.http import HttpRequest, HttpResponse
from django.shortcuts import redirect, reverse
from django.contrib import messages
from django.utils.html import format_html
from django.utils.translation import gettext as _

from django_temp_permissions.models import TemporaryPermission
from django.contrib.auth.admin import UserAdmin
from django.contrib.auth import get_user_model

from django_temp_permissions.signals import permission_overlap_detected


@admin.register(TemporaryPermission)
class TemporaryPermissionAdmin(ExtraButtonsMixin, AdminFiltersMixin, admin.ModelAdmin):
    """Admin interface for TemporaryPermission model.

    Provides a user-friendly interface for managing temporary permissions
    with filtering, searching, and display customization.
    """

    list_display = (
        "user",
        "user__first_name",
        "user__last_name",
        "start_datetime",
        "end_datetime",
    )
    search_fields = ("user__username", "user__email")
    ordering = ("-start_datetime",)

    filter_horizontal = ("permissions",)

    list_filter = (
        ("start_datetime", DateRangeFilter),
        ("end_datetime", DateRangeFilter),
    )

    def _check_overlaps(self, request: HttpRequest, user_temp_permission_id: int) -> None:
        """Inform the user via the admin UI that the permission(s) they assigned."""
        temp_perm = TemporaryPermission.objects.get(id=user_temp_permission_id)
        for overlapping_perm in TemporaryPermission.objects.overlapping(temp_perm):
            url = reverse("admin:django_temp_permissions_temporarypermission_change", args=(overlapping_perm.pk,))
            link = format_html(
                '<a href="{url}" target="_blank"> User Temporary Permission ID {id}</a>',
                url=url,
                id=overlapping_perm.id,
            )

            message = format_html(
                _(
                    "One or more of the permissions you saved are already covered for the specified time period in:{link}"  # noqa: E501
                ),
                link=link,
            )

            self.message_user(request, message, level=messages.WARNING)
            permission_overlap_detected.send(sender=self.__class__, message="Overlaps detected.")

    # FIXME https://github.com/saxix/django-admin-extra-buttons/issues/6, disable when changing form
    @button()
    def check_overlaps(
        self, request: HttpRequest, user_temp_permission_id: int
    ) -> HttpResponse:  # returned by the django-admin-extra buttons
        """Allow user to check for overlaps on demands."""
        self._check_overlaps(request, user_temp_permission_id)

    def save_related(self, request: HttpRequest, form: Any, formsets: Any, change: Any) -> None:  # noqa: ANN401
        """Override, to check for overlaps on every save."""
        super().save_related(request, form, formsets, change)
        self._check_overlaps(request, form.instance.id)


class TemporaryPermissionCustomUserAdmin(ExtraButtonsMixin, UserAdmin):
    """Custom admin for the default User model."""

    @button()
    def configure_temporary_permission(self, request: HttpRequest, obj: AbstractBaseUser) -> HttpResponse:
        """Redirect to TemporaryPermission create view with user pre-filled."""
        url = reverse("admin:django_temp_permissions_temporarypermission_add")
        return redirect(f"{url}?user={obj}")


admin.site.unregister(get_user_model())
admin.site.register(get_user_model(), TemporaryPermissionCustomUserAdmin)



"""Models for django-temp-permissions.

This module provides models for managing temporary permissions in Django,
allowing administrators to grant time-limited permissions to users.
"""

from django.contrib.auth.models import Permission
from django.db import models
from django.db.models import QuerySet
from django.utils import timezone
from django.contrib.auth import get_user_model
from django.core.exceptions import ValidationError
from django.utils.translation import gettext as _

from django_temp_permissions.validators import validate_date_not_in_past

User = get_user_model()  # noqa


class BaseTemporaryPermission(models.Model):
    """Abstract base model for temporary permissions.

    Provides common fields for all temporary permission types including
    time constraints and optional notes.

    Attributes:
        start_datetime: When the temporary permission becomes active
        end_datetime: When the temporary permission expires
        notes: Optional notes about why the permission was granted

    """

    permissions = models.ManyToManyField(
        Permission, related_name="temporary_permissions", help_text=_("The permission being granted temporarily")
    )

    start_datetime = models.DateTimeField(
        validators=[validate_date_not_in_past], help_text=_("Date and time when the permission becomes active")
    )
    end_datetime = models.DateTimeField(
        validators=[validate_date_not_in_past], help_text=_("Date and time when the permission expires")
    )
    notes = models.TextField(blank=True, default="", help_text=_("Optional notes about this temporary permission"))

    class Meta:
        abstract = True


class TemporaryPermissionQuerySet(models.QuerySet):
    """Custom chained queryset for TemporaryPermission."""

    def get_active_permissions(self, user: User) -> QuerySet[Permission]:
        """Permissions granted to a given user."""
        now = timezone.now()
        return Permission.objects.filter(
            temporary_permissions__user=user,
            temporary_permissions__start_datetime__lte=now,
            temporary_permissions__end_datetime__gt=now,
        ).distinct()

    def with_perm(self, permission: Permission | str, content_type_app_label: str | None = None) -> QuerySet[User]:
        """Query for users whose permission is currently active.

        Active means the current time (to the second) lies within
        the start and end datetime specified within the TemporaryPermission
        linked to the permission.

        Args:
            permission: The Django permission being granted temporarily.
            content_type_app_label: The Django content type app label.

        Returns:
            A QuerySet of user(s).

        """
        if isinstance(permission, str):
            permission = Permission.objects.get(content_type__app_label=content_type_app_label, codename=permission)

        now = timezone.now()
        return User.objects.filter(
            temporary_permissions__permissions=permission,
            temporary_permissions__start_datetime__lte=now,
            temporary_permissions__end_datetime__gt=now,
        ).distinct()

    def overlapping(self, temp_perm: 'TemporaryPermission') -> QuerySet['TemporaryPermission']:
        """Find overlapping temporary permissions for the same user, for the same time period.

        Args:
            temp_perm: Either a TempPerm instance or an integer ID

        Returns:
            A QuerySet of TemporaryPermission(s).

        """
        if isinstance(temp_perm, int):
            temp_perm = self.get(id=temp_perm)

        # Get the permission IDs from temp_perm
        permission_ids = temp_perm.permissions.values_list("id", flat=True)

        return (
            self.filter(
                user=temp_perm.user,
                start_datetime__lte=temp_perm.end_datetime,
                end_datetime__gte=temp_perm.start_datetime,
                permissions__id__in=permission_ids,
            )
            .exclude(id=temp_perm.id)
            .distinct()
        )


class TemporaryPermission(BaseTemporaryPermission):
    """Temporary permission granted to a specific user.

    Links a user to a permission with time constraints.
    """

    user = models.ForeignKey(
        get_user_model(),
        on_delete=models.CASCADE,
        related_name="temporary_permissions",
        help_text=_("User receiving the temporary permission"),
    )

    objects = TemporaryPermissionQuerySet().as_manager()

    def __str__(self) -> str:
        """Human-readable representation of the temporary permission."""
        return f"{self.user}: {self.start_datetime} → {self.end_datetime}"

    def clean(self) -> None:
        """Validate temporary permission constraints.

        Additional validations:
        - Validate that at least one permission is assigned.
        - Validate that end_datetime is after start_datetime.
        """
        if self.pk and not self.permissions.exists():
            raise ValidationError(_("At least one Permission must be selected."))

        # Preventing an exception in case of either not being passed
        if self.start_datetime and self.end_datetime and self.end_datetime <= self.start_datetime:
            raise ValidationError(_("End datetime must be after start datetime."))

        super().clean()
